"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sentryIgnoreErrors = exports.createErrorFormatter = exports.ConnectionFailedError = exports.NotAFeedError = exports.ParserError = exports.UnknownRequestError = exports.TimeoutError = exports.ConnectionRefusedError = exports.DnsLookupError = exports.NotFoundError = exports.UpstreamEncryptionError = exports.UpstreamHttpError = exports.InvalidInputError = exports.EmptyHttpResponseError = exports.EmptyParserOutputError = exports.BaseError = void 0;
const http_status_codes_1 = require("http-status-codes");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('graphql-rss-parser:errors');
const development = !process.env['NODE_ENV'] ||
    process.env['NODE_ENV'] === 'development' ||
    process.env['NODE_ENV'] === 'test';
class BaseError extends Error {
    constructor(message, code) {
        super(message);
        this.name = this.constructor.name;
        this.code = code || 'internal-server-error';
        if (typeof Error.captureStackTrace === 'function') {
            Error.captureStackTrace(this, this.constructor);
        }
        else {
            this.stack = new Error(message).stack;
        }
    }
}
exports.BaseError = BaseError;
class EmptyParserOutputError extends BaseError {
    constructor() {
        super('Internal server error', 'empty-parse-output');
    }
}
exports.EmptyParserOutputError = EmptyParserOutputError;
class EmptyHttpResponseError extends BaseError {
    constructor() {
        super('Empty response from feed', 'empty-http-response-output');
    }
}
exports.EmptyHttpResponseError = EmptyHttpResponseError;
class InvalidInputError extends BaseError {
    constructor(message, code) {
        super(message, code || 'invalid-input');
    }
}
exports.InvalidInputError = InvalidInputError;
class UpstreamHttpError extends BaseError {
    constructor(message, status) {
        super(message, 'upstream-http-error');
        this.status = status;
        try {
            this.statusText = (0, http_status_codes_1.getReasonPhrase)(status);
        }
        catch (error) {
            this.statusText = `Unknown error (${status})`;
        }
    }
}
exports.UpstreamHttpError = UpstreamHttpError;
class UpstreamEncryptionError extends BaseError {
    constructor(error) {
        super('Upstream encryption error', 'upstream-encryption-error');
        this.cause = error;
    }
}
exports.UpstreamEncryptionError = UpstreamEncryptionError;
class NotFoundError extends BaseError {
    constructor() {
        super('Could not find feed', 'could-not-find-feed');
    }
}
exports.NotFoundError = NotFoundError;
class DnsLookupError extends BaseError {
    constructor() {
        super('Could not find domain', 'dns-lookup-error');
    }
}
exports.DnsLookupError = DnsLookupError;
class ConnectionRefusedError extends BaseError {
    constructor() {
        super('The website refused the connection', 'connection-refused');
    }
}
exports.ConnectionRefusedError = ConnectionRefusedError;
class TimeoutError extends BaseError {
    constructor() {
        super('The request for the feed timed out', 'timeout');
    }
}
exports.TimeoutError = TimeoutError;
class UnknownRequestError extends BaseError {
    constructor(cause) {
        super('Unknown error while requesting feed', 'unknown-request-error');
        this.cause = cause;
    }
}
exports.UnknownRequestError = UnknownRequestError;
class ParserError extends BaseError {
    constructor(cause, parser) {
        super(cause.message, 'parser-error');
        this.cause = cause;
        this.parser = parser;
    }
}
exports.ParserError = ParserError;
class NotAFeedError extends BaseError {
    constructor(cause) {
        super('Not a feed', 'not-a-feed');
        this.cause = cause;
    }
}
exports.NotAFeedError = NotAFeedError;
class ConnectionFailedError extends BaseError {
    constructor(url) {
        super('Could not connect', 'connection-failed');
        this.url = url;
    }
}
exports.ConnectionFailedError = ConnectionFailedError;
function createErrorFormatter(Sentry) {
    debug(Sentry
        ? 'creating error formatter with sentry'
        : 'creating error formatter without sentry');
    return function formatError(error) {
        const response = {
            path: error.path,
            error: {
                message: error.message,
                code: error.originalError.code,
                url: error.originalError.url,
                status: error.originalError.status,
                statusText: error.originalError.statusText,
                parser: error.originalError.parser,
            },
        };
        if (error.stack) {
            if (development) {
                response.stack = error.stack.split('\n');
            }
            try {
                response.type = error.stack.split('\n')[0].split(':')[0];
            }
            catch (error) {
                if (development) {
                    return error;
                }
            }
        }
        if (error.extensions?.exception?.stacktrace) {
            console.error(error.extensions?.exception?.stacktrace);
            if (development) {
                response.stack = error.extensions?.exception?.stacktrace;
            }
            try {
                response.type =
                    error.extensions?.exception?.stacktrace[0].split(':')[0];
            }
            catch (error) {
                if (development) {
                    return error;
                }
            }
        }
        debug.extend('formatError')('error response', response);
        if (Sentry) {
            Sentry.captureException(error.originalError || error, {
                extra: {
                    path: error.path,
                    apiResponse: response,
                },
            });
        }
        return response;
    };
}
exports.createErrorFormatter = createErrorFormatter;
exports.sentryIgnoreErrors = [
    'ConnectionFailedError',
    'ConnectionRefusedError',
    'DnsLookupError',
    'EmptyHttpResponseError',
    'EmptyParseOutputError',
    'InvalidInputError',
    'NotAFeedError',
    'NotFoundError',
    'ParserError',
    'TimeoutError',
    'UpstreamHttpError',
    'UpstreamEncryptionError',
    'ValidationError',
];
//# sourceMappingURL=errors.js.map