"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findFeed = exports.normalizeFeedLink = void 0;
const cheerio_1 = __importDefault(require("cheerio"));
const normalize_url_1 = __importDefault(require("normalize-url"));
const feed_1 = require("./feed");
const errors_1 = require("../errors");
const request_1 = __importDefault(require("../request"));
const normalizeOptions = { removeTrailingSlash: false, stripHash: true };
function normalizeFeedLink(baseUrl, link) {
    return (0, normalize_url_1.default)(link && /^http/.test(link)
        ? link
        : new URL(baseUrl).origin +
            (link && /^\//.test(link) ? link : `/${link}`), normalizeOptions);
}
exports.normalizeFeedLink = normalizeFeedLink;
function mapLinkTagToUrl(normalizedUrl) {
    return (linkTag) => {
        return normalizeFeedLink(normalizedUrl, linkTag.attribs['href']);
    };
}
async function findJsonFeedsInDom(dom, normalizedUrl) {
    const linkTags = dom('link[rel="alternate"][type="application/feed+json"]');
    const urls = linkTags.toArray().map(mapLinkTagToUrl(normalizedUrl));
    return (await Promise.all(urls.map(async (url) => {
        try {
            const { title } = await (0, feed_1.parseFromQuery)({
                url,
                parser: 'JSON_FEED_V1',
            });
            return { title, link: url };
        }
        catch (error) {
            if (process.env['NODE_ENV'] !== 'production' &&
                process.env['NODE_ENV'] !== 'test') {
                console.log(error);
            }
            console.error(url, error);
            return undefined;
        }
    }))).filter((item) => !!item);
}
async function findRssFeedsInDom(dom, normalizedUrl) {
    const linkTags = dom('link[rel="alternate"][type="application/rss+xml"]').add('link[rel="alternate"][type="application/atom+xml"]');
    const urls = linkTags.toArray().map(mapLinkTagToUrl(normalizedUrl));
    return (await Promise.all(urls.map(async (url) => {
        try {
            const { title } = await (0, feed_1.parseFromQuery)({ url });
            return { title, link: url };
        }
        catch (error) {
            if (process.env['NODE_ENV'] !== 'production' &&
                process.env['NODE_ENV'] !== 'test') {
                console.log(error);
            }
            return undefined;
        }
    }))).filter((item) => !!item);
}
async function findFeed({ url, normalize = false, }) {
    const normalizedUrl = normalize ? url : (0, normalize_url_1.default)(url, normalizeOptions);
    if (!normalizedUrl) {
        throw new errors_1.BaseError('Empty url is not allowed', 'missing-url');
    }
    const response = await (0, request_1.default)(normalizedUrl);
    const content = response.text;
    if (/application\/(rss|atom)/.test(response.contentType || '') ||
        /(application|text)\/xml/.test(response.contentType || '')) {
        try {
            const { title } = await (0, feed_1.parseFromString)({ content });
            return [{ title, link: normalizedUrl }];
        }
        catch (error) {
            if (process.env['NODE_ENV'] !== 'production' &&
                process.env['NODE_ENV'] !== 'test') {
                console.log(error);
            }
        }
    }
    if (/application\/feed\+json/.test(response.contentType || '') ||
        /application\/json/.test(response.contentType || '')) {
        try {
            const { title } = await (0, feed_1.parseFromQuery)({
                url,
                parser: 'JSON_FEED_V1',
            });
            return [{ title, link: normalizedUrl }];
        }
        catch (error) {
            if (process.env['NODE_ENV'] !== 'production' &&
                process.env['NODE_ENV'] !== 'test') {
                console.log(error);
            }
        }
    }
    const dom = cheerio_1.default.load(response.text);
    if (dom('rss')) {
        try {
            const { title } = await (0, feed_1.parseFromString)({ content });
            return [{ title, link: normalizedUrl }];
        }
        catch (error) {
            if (process.env['NODE_ENV'] !== 'production' &&
                process.env['NODE_ENV'] !== 'test') {
                console.log(error);
            }
        }
    }
    const result = [
        ...(await findRssFeedsInDom(dom, normalizedUrl)),
        ...(await findJsonFeedsInDom(dom, normalizedUrl)),
    ];
    if (result.length === 0 && normalize) {
        return findFeed({ url, normalize: false });
    }
    return result;
}
exports.findFeed = findFeed;
//# sourceMappingURL=findFeed.js.map