"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const debug_1 = __importDefault(require("debug"));
const stream_1 = require("stream");
const feedme_1 = __importDefault(require("feedme"));
const errors_1 = require("../errors");
const debug = (0, debug_1.default)('graphql-rss-parser:parsers:feedme');
const findHtmlLink = (array) => {
    const link = array.find((item) => typeof item === 'object' &&
        item['rel'] === 'alternate' &&
        item['type'] === 'text/html');
    if (typeof link === 'object' && typeof link?.['href'] === 'string') {
        return link?.['href'] || undefined;
    }
    return undefined;
};
function evaluateLink(link) {
    if (Array.isArray(link)) {
        const htmlLink = findHtmlLink(link);
        if (htmlLink) {
            return htmlLink;
        }
    }
    if (typeof link === 'string') {
        return link;
    }
    throw new errors_1.ParserError(new Error('Missing link'), 'FEEDME');
}
function unpack(input, attribute, required, key) {
    let output = undefined;
    if (input && Array.isArray(input)) {
        // eslint-disable-next-line @typescript-eslint/no-use-before-define
        output = unpackArray(input, attribute)[0];
    }
    if (typeof input === 'string') {
        output = input;
    }
    // @ts-ignore ---
    if (typeof input === 'object' && typeof input[attribute] === 'string') {
        // @ts-ignore ---
        output = input[attribute];
    }
    if (required && !output) {
        throw new errors_1.ParserError(new Error(`Missing field ${key}`), 'FEEDME');
    }
    return output;
}
function unpackArray(input, attribute) {
    if (Array.isArray(input)) {
        return input
            .map((item) => unpack(item, attribute))
            .filter((item) => !!item);
    }
    if (typeof input === 'string') {
        const unpacked = unpack(input, attribute);
        return unpacked ? [unpacked] : [];
    }
    return [];
}
function parse(feed) {
    return new Promise((resolve, reject) => {
        debug('starting to parse');
        try {
            if (feed.includes('medium.com')) {
                throw new Error('Failed to parse');
            }
            const parser = new feedme_1.default(true);
            parser.on('end', () => {
                const parsed = parser.done();
                if (!parsed) {
                    return reject(new errors_1.EmptyParserOutputError());
                }
                debug('done parsing');
                try {
                    resolve({
                        parser: 'FEEDME',
                        title: unpack(parsed['title'], 'text', true, 'title'),
                        description: unpack(parsed['description'], 'text'),
                        home_page_url: evaluateLink(parsed['link']),
                        feed_url: undefined,
                        items: parsed.items.map((item) => {
                            const pubDate = unpack(item['pubdate'], 'text');
                            return {
                                title: unpack(item['title'], 'text'),
                                url: evaluateLink(item['link']),
                                id: unpack(item['id'] || item['guid'], 'text', true, 'id'),
                                content_html: unpack(item['description'], 'text'),
                                tags: unpackArray(item['category'], 'text'),
                                date_published: pubDate
                                    ? new Date(pubDate).toISOString()
                                    : pubDate,
                                authors: unpack(item['author'], 'name')
                                    ? [{ name: unpack(item['author'], 'name') }]
                                    : [],
                            };
                        }),
                    });
                }
                catch (error) {
                    reject(new errors_1.ParserError(error, 'FEEDME'));
                }
            });
            parser.on('error', (error) => {
                debug('parsing failed with error', error);
                reject(new errors_1.ParserError(error, 'FEEDME'));
            });
            const stream = new stream_1.Readable();
            stream.pipe(parser);
            stream.push(feed);
            stream.push(null);
        }
        catch (error) {
            debug('parsing failed with error', error);
            reject(error);
        }
    });
}
exports.parse = parse;
//# sourceMappingURL=feedme.js.map